@@ -25,6 +25,7 @@
 #include <sys/stat.h>
 #include <sys/fcntl.h>
 
+#include <errno.h>
 #include <string.h>
 #include <unistd.h>
 #include <libedataserver/e-time-utils.h>
@@ -71,7 +72,7 @@ struct ECalBackendExchangePrivate {
 #define PARENT_TYPE E_TYPE_CAL_BACKEND_SYNC
 static GObjectClass *parent_class = NULL;
 
-#define d(x) x 
+#define d(x) 
 
 static ECalBackendSyncStatus
 is_read_only (ECalBackendSync *backend, EDataCal *cal, gboolean *read_only)
@@ -97,6 +98,18 @@ get_cal_address (ECalBackendSync *backend, EDataCal *cal, char **address)
 	return GNOME_Evolution_Calendar_Success;
 }
 
+static void
+get_cal_owner (ECalBackendSync *backend, char **name)
+{
+	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
+	ExchangeHierarchy *hier;
+
+	g_return_if_fail (E_IS_CAL_BACKEND_EXCHANGE (cbex));
+
+	hier = e_folder_exchange_get_hierarchy (cbex->folder);
+	*name = g_strdup (hier->owner_name);
+}
+
 static ECalBackendSyncStatus
 get_alarm_email_address (ECalBackendSync *backend, EDataCal *cal, char **address)
 {
@@ -170,7 +183,8 @@ load_cache (ECalBackendExchange *cbex, E2kUri *e2kuri)
 	end = strrchr (cbex->priv->object_cache_file, '/');
 	storage_dir = g_strndup (cbex->priv->object_cache_file, end - cbex->priv->object_cache_file);
 	if (lstat(cbex->priv->local_attachment_store , &buf) < 0) {
-		symlink (storage_dir, cbex->priv->local_attachment_store);
+		if (symlink (storage_dir, cbex->priv->local_attachment_store) < 0)
+			g_warning ("%s: symlink() failed: %s", G_STRFUNC, g_strerror (errno));
 	}
 	g_free (storage_dir);
 	g_free (mangled_uri);
@@ -369,26 +383,53 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 	cbex->account = exchange_component_get_account_for_uri (global_exchange_component, uristr);
 	if (!cbex->account) {
 		cbex->account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
+		exchange_account_set_online (cbex->account);
 		if (!exchange_account_connect (cbex->account, password, &acresult)) {
 			g_mutex_unlock (cbex->priv->open_lock);
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbex), _("Authentication failed"));
 			return GNOME_Evolution_Calendar_AuthenticationFailed;
 		}
 	}
 
-	if (!exchange_account_get_context (cbex->account)) {
+	if (exchange_account_get_context (cbex->account)) {
+		exchange_account_set_online (cbex->account);
 		if(!exchange_account_connect (cbex->account, password, &acresult)) {
 			g_mutex_unlock (cbex->priv->open_lock);
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbex), _("Authentication failed"));
 			return GNOME_Evolution_Calendar_AuthenticationFailed;
 		}
 	}
 
 	cbex->folder = exchange_account_get_folder (cbex->account, uristr);
 	if (!cbex->folder) {
+		ESource *source;
+		const char *foreign; 		
 		/* FIXME: theoretically we should create it if
 		 * only_if_exists is FALSE.
 		 */
-		g_mutex_unlock (cbex->priv->open_lock);
-		return GNOME_Evolution_Calendar_NoSuchCal;
+		
+		source = e_cal_backend_get_source (E_CAL_BACKEND (cbex));
+		foreign = e_source_get_property (source, "foreign");
+
+		if (foreign && (g_str_equal (foreign, "1"))) {
+			char **split_path;
+			const char *email, *path;
+
+			path = strrchr (uristr, ';');
+			split_path = g_strsplit (++path, "/", -1);
+			email = split_path [0];
+			
+			exchange_account_scan_foreign_hierarchy (cbex->account, email);
+			cbex->folder = exchange_account_get_folder (cbex->account, uristr);
+			
+			g_strfreev (split_path);
+		} else
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbex), _("Could not find the calendar"));
+	
+		if (!cbex->folder) {
+			g_mutex_unlock (cbex->priv->open_lock);
+			return GNOME_Evolution_Calendar_NoSuchCal;
+		}
 	}
 	g_object_ref (cbex->folder);
 
@@ -433,11 +474,11 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 static ECalBackendSyncStatus
 remove_calendar (ECalBackendSync *backend, EDataCal *cal)
 {
-	d(printf("ecbe_remove_calendar(%p, %p)\n", backend, cal));
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
 	ExchangeAccountFolderResult result;
 	const char *uri;
 
+	d(printf("ecbe_remove_calendar(%p, %p)\n", backend, cal));
 	uri = e_folder_exchange_get_internal_uri (cbex->folder);
 	result = exchange_account_remove_folder (cbex->account, uri);
 	if (result == EXCHANGE_ACCOUNT_FOLDER_OK)
@@ -1168,7 +1209,7 @@ set_default_timezone (ECalBackendSync *backend, EDataCal *cal,
 static void
 start_query (ECalBackend *backend, EDataCalView *view)
 {
-	char *sexp = NULL;
+	const char *sexp = NULL;
 	ECalBackendSyncStatus status = GNOME_Evolution_Calendar_OtherError;
 	GList *m, *objects = NULL;
 
@@ -1329,18 +1370,6 @@ e_cal_backend_exchange_lf_to_crlf (const char *in)
 	return out;
 }
 
-static void
-get_cal_owner (ECalBackendSync *backend, char **name)
-{
-	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
-	ExchangeHierarchy *hier;
-
-	g_return_if_fail (E_IS_CAL_BACKEND_EXCHANGE (cbex));
-
-	hier = e_folder_exchange_get_hierarchy (cbex->folder);
-	*name = g_strdup (hier->owner_name);
-}
-
 void
 e_cal_backend_exchange_get_from (ECalBackendSync *backend, ECalComponent *comp, 
 			char **name, char **email)
@@ -1360,6 +1389,19 @@ e_cal_backend_exchange_get_from (ECalBackendSync *backend, ECalComponent *comp,
 	}
 }
 
+void
+e_cal_backend_exchange_get_sender (ECalBackendSync *backend, ECalComponent *comp, 
+			char **name, char **email)
+{
+	ECalBackendExchange *cbex;
+
+	g_return_if_fail (E_IS_CAL_BACKEND_EXCHANGE (backend));
+
+	cbex = E_CAL_BACKEND_EXCHANGE (backend);
+
+	*name = g_strdup (exchange_account_get_username (cbex->account));
+	*email = g_strdup (exchange_account_get_email_id (cbex->account));
+}
 
 /* Do not internationalize */
 static const char *e2k_rfc822_months [] = {
@@ -1404,6 +1446,18 @@ e_cal_backend_exchange_get_from_string (ECalBackendSync *backend, ECalComponent
 	return from_string;
 }
 
+char *
+e_cal_backend_exchange_get_sender_string (ECalBackendSync *backend, ECalComponent *comp)
+{
+	char *name = NULL, *addr = NULL, *sender_string = NULL;
+
+	e_cal_backend_exchange_get_sender (backend, comp, &name, &addr);
+	sender_string = g_strdup_printf ("\"%s\" <%s>", name, addr);
+	g_free (name);
+	g_free (addr);
+	return sender_string;
+}
+
 gchar *
 e_cal_backend_exchange_get_owner_email (ECalBackendSync *backend)
 {
@@ -1414,6 +1468,16 @@ e_cal_backend_exchange_get_owner_email (ECalBackendSync *backend)
 	return g_strdup (hier->owner_email);
 }
 
+gchar *
+e_cal_backend_exchange_get_owner_name (ECalBackendSync *backend)
+{
+	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
+	ExchangeHierarchy *hier;
+
+	hier = e_folder_exchange_get_hierarchy (cbex->folder);
+	return g_strdup (hier->owner_name);
+}
+
 struct ChangeData {
 	EXmlHash *ehash;
 	GList *adds;
@@ -1483,7 +1547,7 @@ e_cal_backend_exchange_compute_changes_foreach_key (const char *key, const char
 	ECalBackendExchangeComponent *ecomp;
 	ecomp = g_hash_table_lookup (cbedata->cbex->priv->objects, key);
 
-	if (!ecomp) {
+	if (ecomp) {
 		ECalComponent *comp;
 		comp = e_cal_component_new ();
 		if (ecomp->icomp)
@@ -1568,7 +1632,7 @@ get_attachment (ECalBackendExchange *cbex, const char *uid,
 				attach_data = g_memdup (stream_mem->buffer->data, stream_mem->buffer->len);
 				attach_file = g_strdup_printf ("%s/%s-%s", cbex->priv->local_attachment_store, uid, filename);
 				// Attach
-				attach_file_url = save_attach_file (attach_file, attach_data, stream_mem->buffer->len);
+				attach_file_url = save_attach_file (attach_file, (char *) attach_data, stream_mem->buffer->len);
 				g_free (attach_data);
 				g_free (attach_file);
 				d(printf ("attach file name : %s\n", attach_file_url));
@@ -1656,6 +1720,8 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 	e_cal_component_get_uid (comp, &uid);
 	e_cal_component_get_attachment_list (comp, &attach_list);
 	for (l = attach_list; l ; l = l->next){
+		char *mime_type;
+
 		if (!strncmp ((char *)l->data, "file://", 7)) {
 			fname = (char *)(l->data) + strlen ("file://");
 			filename = g_strrstr (fname, "/") + 1;
@@ -1663,7 +1729,15 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 			attach_file = g_strdup (fname);
 		} else {
 			fname = (char *)(l->data);
-			filename = g_strrstr (fname, "/") + 1;
+			filename = g_strrstr (fname, "/");
+			if (!filename) {
+				/*
+				 * some vcards contain e.g. ATTACH:CID:0fd601c67efb$ef66760d$_CDOEX
+				 * -> ignore instead of crashing
+				 */
+				continue;
+			}
+			filename++;
 			mime_filename = filename;
 			attach_file = g_strdup_printf ("%s/%s-%s", cbex->priv->local_attachment_store, uid, filename);
 		}
@@ -1684,7 +1758,7 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 		camel_data_wrapper_construct_from_stream (wrapper, stream);
 		camel_object_unref (stream);
 
-		char *mime_type = gnome_vfs_get_mime_type (dest_url + strlen ("file://"));
+		mime_type = gnome_vfs_get_mime_type (dest_url + strlen ("file://"));
 		type = camel_content_type_decode (mime_type);
 		camel_data_wrapper_set_mime_type_field (wrapper, type);
 		camel_content_type_unref (type);
